source("dataS/scripts/vehicles/specializations/SetTurnedOnEvent.lua");
source("dataS/scripts/vehicles/specializations/CombineSetStrawEnableEvent.lua");

ForageWagon = {};

function ForageWagon.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function ForageWagon:load(xmlFile)

    self.setIsTurnedOn = SpecializationUtil.callSpecializationsFunction("setIsTurnedOn");

    local forageWgnSound = getXMLString(xmlFile, "vehicle.forageWgnSound#file");
    if forageWgnSound ~= nil and forageWgnSound ~= "" then
		forageWgnSound = Utils.getFilename(forageWgnSound, self.baseDirectory);
        self.forageWgnSound = createSample("forageWgnSound");
        self.forageWgnSoundEnabled = false;
        loadSample(self.forageWgnSound, forageWgnSound, false);
        self.forageWgnSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#pitchOffset"), 1);
        self.forageWgnSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#volume"), 1.0);
    end;
  
	self.pickupAnimationName = Utils.getNoNil(getXMLString(xmlFile, "vehicle.pickupAnimation#name"), "");
	if self.playAnimation == nil or self.getIsAnimationPlaying == nil then
		self.pickupAnimationName = "";
    end;
    self.pickupAnimationLowerSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.pickupAnimation#lowerSpeed"), 1);
    self.pickupAnimationLiftSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.pickupAnimation#liftSpeed"), -self.pickupAnimationLowerSpeed);

    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);
    self.wasToFast = false;
    self.isTurnedOn = false;

    self.lastForageWagonArea = 0;
	self.cutterWheels = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.cutterWheels#index"));
    
	self.forageWagonGroundFlag = self:getNextDirtyFlag();
	
		-- UV Scroll for the elevator --
    self.ElevatorUVScrollParts = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.ElevatorUVScrollParts.ElevatorUVScrollPart(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
			break;
        end;
        local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local speed = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#speed"), 2);
         if node ~= nil and speed then
              table.insert(self.ElevatorUVScrollParts, {node=node, speed=speed});
        end;
        i = i +1;
    end;	
	-- UV Scroll for the beets elevator --
    self.BeetsElevatorUVScrollParts = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.BeetsElevatorUVScrollParts.BeetsElevatorUVScrollPart(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
			break;
        end;
        local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local speed = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#speed"), 2);
         if node ~= nil and speed then
              table.insert(self.BeetsElevatorUVScrollParts, {node=node, speed=speed});
        end;
        i = i +1;
    end;	

	self.strawParticleSystems = {};
    local i = 0;
    while true do
		local key = string.format("vehicle.strawParticleSystems.strawParticleSystem(%d)", i);
        local t = getXMLString(xmlFile, key .. "#type");
        if t == nil then
			break;
		end;
        local desc = FruitUtil.fruitTypes[t];
        if desc ~= nil then
			local currentPS = {};
            local particleNode = Utils.loadParticleSystem(xmlFile, currentPS, key, self.components, false, "$data/vehicles/particleSystems/threshingStrawParticleSystem.i3d", self.baseDirectory);
            self.strawParticleSystems[desc.index] = currentPS;
            if self.defaultStrawParticleSystem == nil then
				self.defaultStrawParticleSystem = currentPS;
			end;
		end;
        i = i + 1;
    end;
	self.strawToggleTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.strawParticleSystems#toggleTime"), 2500);
    self.strawEnableTime = nil;
    self.strawDisableTime = nil;
    self.strawEmitState = false;	
end;

function ForageWagon:delete()
    if self.forageWgnSound ~= nil then
        delete(self.forageWgnSound);
    end;
	for k,v in pairs(self.strawParticleSystems) do
		Utils.deleteParticleSystem(v);
	end;
end;

function ForageWagon:readStream(streamId, connection)
    local turnedOn = streamReadBool(streamId);
    self:setIsTurnedOn(turnedOn, true);
    local fillLevel = streamReadInt32(streamId);
    self:setFillLevel(fillLevel, Fillable.FILLTYPE_SUGARBEET);
	CombineSetStrawEnableEvent.execute(self, strawPSenabled, strawPSFruitType);
end;

function ForageWagon:writeStream(streamId, connection)
    streamWriteBool(streamId, self.isTurnedOn);
    streamWriteInt32(streamId, self.fillLevel);
end;

function ForageWagon:readUpdateStream(streamId, timestamp, connection)
end;

function ForageWagon:writeUpdateStream(streamId, connection, dirtyMask)
end;

function ForageWagon:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ForageWagon:keyEvent(unicode, sym, modifier, isDown)
end;

function ForageWagon:update(dt)

    if self:getIsActiveForInput() then
        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
            self:setIsTurnedOn(not self.isTurnedOn);
        end;
    end;
end;

function ForageWagon:updateTick(dt)
    self.wasToFast = false;
    self.lastForageWagonArea = 0;
    if self:getIsActive() then
        if self.isTurnedOn and self:allowFillType(Fillable.FILLTYPE_SUGARBEET) and self.capacity > self.fillLevel then
			rotate(self.cutterWheels, 0.2, 0, 0);
            local toFast = self:doCheckSpeedLimit() and self.attacherVehicle.lastSpeed*3600 > 29;
            if self.isServer then
                if not toFast then
                    local cuttingAreasSend = {};
                    for k, cuttingArea in pairs(self.cuttingAreas) do
                        if self:getIsAreaActive(cuttingArea) then
                            local x,y,z = getWorldTranslation(cuttingArea.start);
                            local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
                            local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

                            table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2});
                        end;
                    end;
                    if (table.getn(cuttingAreasSend) > 0) then
						local lastArea, realArea = SugarBeetCutterAreaEvent.runLocally(cuttingAreasSend, FruitUtil.FRUITTYPE_SUGARBEET);
						self.lastArea = lastArea;
						self.lastAreaBiggerZero = (self.lastArea > 0);
						if self.lastArea > 0 then
							local pixelToSqm = g_currentMission:getFruitPixelsToSqm(); -- 4096px are mapped to 2048m
							local sqm = realArea*pixelToSqm;
							local literPerSqm = g_strawLitersPerSqm * 2;
							local deltaLevel = sqm*literPerSqm * self.fillScale;
							self:setFillLevel(self.fillLevel+deltaLevel, Fillable.FILLTYPE_SUGARBEET);
							g_server:broadcastEvent(SugarBeetCutterAreaEvent:new(cuttingAreasSend, FruitUtil.FRUITTYPE_SUGARBEET));
							g_server:broadcastEvent(CombineSetStrawEnableEvent:new(self, true, FruitUtil.FRUITTYPE_SUGARBEET), true, nil, self);
							for k,v in pairs(self.strawParticleSystems) do
								Utils.setEmittingState(v, true);
							end;
							self.strawEmitState = true;
							self:setState("work:1", true);
							self:setState("work:2", true);
						else
							self:setState("work:1", false);
							self:setState("work:2", false);
							g_server:broadcastEvent(CombineSetStrawEnableEvent:new(self, false, FruitUtil.FRUITTYPE_SUGARBEET), true, nil, self);
							for k,v in pairs(self.strawParticleSystems) do
								Utils.setEmittingState(v, false);
							end;
							self.strawEmitState = false;
						end;									
					else
						self:setState("work:1", false);
						self:setState("work:2", false);
						g_server:broadcastEvent(CombineSetStrawEnableEvent:new(self, false, FruitUtil.FRUITTYPE_SUGARBEET), true, nil, self);
						for k,v in pairs(self.strawParticleSystems) do
							Utils.setEmittingState(v, false);
						end;
						self.strawEmitState = false;
					end;
                else
					self:setState("work:1", false);
					self:setState("work:2", false);
					g_server:broadcastEvent(CombineSetStrawEnableEvent:new(self, false, FruitUtil.FRUITTYPE_SUGARBEET), true, nil, self);
					for k,v in pairs(self.strawParticleSystems) do
						Utils.setEmittingState(v, false);
					end;
					self.strawEmitState = false;
				end;
            end;
			
			for _, part in pairs(self.ElevatorUVScrollParts) do
				setShaderParameter(part.node, "uvScrollSpeed", part.speed[1], part.speed[2], 0, 0, false);
			end;
			for _, part in pairs(self.BeetsElevatorUVScrollParts) do
				setShaderParameter(part.node, "uvScrollSpeed", part.speed[1], part.speed[2], 0, 0, false);
			end;			

			self.wasToFast = toFast;

            if self.isClient then
                if not self.forageWgnSoundEnabled and self:getIsActiveForSound() then
                    playSample(self.forageWgnSound, 0, self.forageWgnSoundVolume, 0);
                    setSamplePitch(self.forageWgnSound, self.forageWgnSoundPitchOffset);
                    self.forageWgnSoundEnabled = true;
                end;
				for k,v in pairs(self.strawParticleSystems) do
					Utils.setEmittingState(v, self.strawEmitState);
				end;
            end;
			for k,v in pairs(self.strawParticleSystems) do
				Utils.setEmittingState(v, self.strawEmitState);
			end;
		else
			self:setState("work:1", false);
			self:setState("work:2", false);
			
			for _, part in pairs(self.ElevatorUVScrollParts) do
				setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
			end;
			for _, part in pairs(self.BeetsElevatorUVScrollParts) do
				setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
			end;
			if self.isServer then
				g_server:broadcastEvent(CombineSetStrawEnableEvent:new(self, false, FruitUtil.FRUITTYPE_SUGARBEET), true, nil, self);
			end;
			for k,v in pairs(self.strawParticleSystems) do
				Utils.setEmittingState(v, false);
			end;
			self.strawEmitState = false;
		end;

		if self.isClient then
            if self.forageWgnSoundEnabled and not self.isTurnedOn then
                stopSample(self.forageWgnSound);
                self.forageWgnSoundEnabled = false;
            end;
			for k,v in pairs(self.strawParticleSystems) do
				Utils.setEmittingState(v, self.strawEmitState);
			end;
        end;
    end;
end;

 function ForageWagon:draw()
    if self.wasToFast then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getKeyNamesOfDigitalAction(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
    end;
    if self.isTurnedOn then
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
    else
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
    end;
end;

function ForageWagon:onDetach()
    if self.deactivateOnDetach then
        ForageWagon.onDeactivate(self);
    else
        ForageWagon.onDeactivateSounds(self);
    end;
end;

function ForageWagon:onLeave()
    if self.deactivateOnLeave then
        ForageWagon.onDeactivate(self);
    else
        ForageWagon.onDeactivateSounds(self);
    end;
end;

function ForageWagon:onDeactivate()
    self.isTurnedOn = false;
    ForageWagon.onDeactivateSounds(self);
	for k,v in pairs(self.strawParticleSystems) do
		Utils.setEmittingState(v, false);
	end;
	self.strawEnableTime = nil;
    self.strawDisableTime = nil;
    self.strawEmitState = false;
	self:setState("work:1", false);
	self:setState("work:2", false);
	for _, part in pairs(self.ElevatorUVScrollParts) do
		setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
	end;
	for _, part in pairs(self.BeetsElevatorUVScrollParts) do
		setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
	end;
end;

function ForageWagon:setIsTurnedOn(isTurnedOn, noEventSend)
    SetTurnedOnEvent.sendEvent(self, isTurnedOn, noEventSend)
    self.isTurnedOn = isTurnedOn;

    if self.pickupAnimationName ~= "" then
        local animTime = nil;
        if self:getIsAnimationPlaying(self.pickupAnimationName) then
            animTime = self:getAnimationTime(self.pickupAnimationName);
        end;
        if isTurnedOn then
            self:playAnimation(self.pickupAnimationName, self.pickupAnimationLowerSpeed, animTime, true)
        else
            self:playAnimation(self.pickupAnimationName, self.pickupAnimationLiftSpeed, animTime, true)
        end;
    end;
end;
 
function ForageWagon:onDeactivateSounds()
    if self.forageWgnSoundEnabled then
        stopSample(self.forageWgnSound);
        self.forageWgnSoundEnabled = false;
    end;
end;